from typing import Annotated, Any

from litestar import Controller, get
from litestar.plugins.pydantic import PydanticDTO
from sparkden.models.shared import BaseModel
from sparkden.shared.utils import dto_config


class PartialAssistant(BaseModel):
    id: str
    disabled: bool
    data_sources: list[dict[str, Any]]


class ListAssistantsResponse(BaseModel):
    assistants: list[PartialAssistant]


ListAssistantsResponseDTO = PydanticDTO[
    Annotated[
        ListAssistantsResponse,
        dto_config(),
    ]
]


class AssistantController(Controller):
    path = "/assistants"

    @get("/", return_dto=ListAssistantsResponseDTO, sync_to_thread=True)
    def list_assistants(self) -> ListAssistantsResponse:
        from sparkden.assistants import assistants

        return ListAssistantsResponse(
            assistants=[
                PartialAssistant(
                    id=assistant.id,
                    disabled=assistant.disabled,
                    data_sources=[
                        data_source.model_dump(
                            exclude={"data_fetcher", "data_splitter"},
                            exclude_none=True,
                            by_alias=True,
                        )
                        for data_source in assistant.data_sources or []
                    ],
                )
                for assistant in assistants.get_assistants(include_disabled=True)
            ]
        )



from sparkden.db.schema import DataSourceStatus, DataSourceType
from sparkden.db.schema import KnowledgeDataSource as StorageDataSource
from sparkden.models.knowledge import (
    DataChunk,
    KnowledgeCollection,
    KnowledgeDataSource,
    KnowledgeDataSourceCreate,
)
from sparkden.models.shared import OffsetPagination, OrderBy
from sparkden.services.base import BaseService
from sparkden.shared.minio import object_exists
from sqlalchemy import delete, func, select
from sqlalchemy.dialects.postgresql import insert
from sqlalchemy.ext.asyncio import AsyncSession

from .collection import get_collection
from .data_loaders.file_loader import FileLoader
from .vector_store import (
    FieldCondition,
    Filter,
    MatchValue,
    VectorStore,
)


class KnowledgeDataSourceService(BaseService):
    async def list_data_sources(
        self, collection_id: str, *, limit: int = 30, offset: int = 0
    ) -> OffsetPagination[KnowledgeDataSource]:
        async with self.db_session_maker.begin() as db_session:
            query = (
                select(
                    StorageDataSource,
                    func.count().over().label("total_count"),
                )
                .where(StorageDataSource.collection_id == collection_id)
                .order_by(StorageDataSource.created_at.desc())
                .offset(offset)
                .limit(limit)
            )
            result = (await db_session.execute(query)).all()
            data_sources = [KnowledgeDataSource.from_storage(row[0]) for row in result]
            total = result[0][1] if result else 0
            return OffsetPagination(
                items=data_sources,
                total=total,
                limit=limit,
                offset=offset,
            )

    async def get_data_source(self, data_source_id: str) -> KnowledgeDataSource | None:
        async with self.db_session_maker.begin() as db_session:
            data_source = await db_session.get(StorageDataSource, data_source_id)
            if data_source:
                return KnowledgeDataSource.from_storage(data_source)
            return None

    async def get_data_source_by_file_hash(
        self,
        collection_id: str,
        file_hash: str,
    ) -> KnowledgeDataSource | None:
        async with self.db_session_maker.begin() as db_session:
            query = select(StorageDataSource).where(
                StorageDataSource.collection_id == collection_id,
                StorageDataSource.extra_info["file_hash"].astext == file_hash,
            )
            data_source = await db_session.scalar(query)
            if data_source:
                return KnowledgeDataSource.from_storage(data_source)
            return None

    async def list_data_source_chunks(
        self,
        collection_id: str,
        data_source_id: str,
        *,
        limit: int = 20,
        offset: int = 0,
    ) -> OffsetPagination[DataChunk]:
        filter = Filter(
            must=[
                FieldCondition(
                    key="data_source_id",
                    match=MatchValue(value=data_source_id),
                ),
            ]
        )

        vector_store = VectorStore(
            collection_name=collection_id,
        )

        chunks = await vector_store.filter_chunks(
            filter,
            limit,
            offset,
            OrderBy(field="sequence_in_data_source", desc=False),
        )

        items = [
            DataChunk(
                id=chunk.id,
                content=chunk.content,
                metadata=chunk.metadata,
            )
            for chunk in chunks
        ]

        return OffsetPagination(
            items=items,
            total=await vector_store.count_chunks(filter),
            limit=limit,
            offset=offset,
        )

    async def add_data_source(
        self, data_source: KnowledgeDataSourceCreate
    ) -> tuple[KnowledgeDataSource, bool]:
        collection = await get_collection(data_source.collection_id)
        if not collection:
            raise ValueError("Collection not found")

        if data_source.type == DataSourceType.UPLOAD:
            return await self.add_file_data_source(data_source, collection)
        else:
            raise NotImplementedError("Not implemented")

    async def add_file_data_source(
        self, data_source: KnowledgeDataSourceCreate, collection: KnowledgeCollection
    ) -> tuple[KnowledgeDataSource, bool]:
        file = data_source.pop_extra_info("file")
        if not file:
            raise ValueError("File not found")

        # prevent duplicate file data source
        if object_exists(data_source.collection_id, f"{file.hash}/original"):
            existing_data_source = await self.get_data_source_by_file_hash(
                data_source.collection_id, file.hash
            )
            if existing_data_source:
                return existing_data_source, False
            else:
                raise ValueError("File not found")

        async with self.db_session_maker.begin() as db_session:
            data_source.set_extra_info("file_hash", file.hash)
            data_source.set_extra_info("file_type", file.content_type)
            new_data_source = await create_data_source(data_source, db_session)

        loader = FileLoader(
            data_source=new_data_source.model_copy(
                update={
                    "extra_info": {
                        **(new_data_source.extra_info or {}),
                        "file": file,
                    }
                }
            ),
            collection=collection,
        )
        await loader.load()

        return new_data_source, True

    async def delete_data_source(self, data_source_id: str) -> None:
        async with self.db_session_maker.begin() as db_session:
            data_source = await db_session.scalar(
                delete(StorageDataSource)
                .where(
                    StorageDataSource.id == data_source_id,
                )
                .returning(StorageDataSource)
            )
        if not data_source:
            return

        if data_source.type == DataSourceType.UPLOAD:
            loader = FileLoader(
                data_source=KnowledgeDataSource.from_storage(data_source),
            )
            await loader.unload()
        else:
            raise NotImplementedError("Not implemented")

    async def refresh_data_source(self, data_source_id: str) -> None:
        data_source = await self.get_data_source(data_source_id)
        if not data_source:
            raise ValueError("Data source not found")

        collection = await get_collection(data_source.collection_id)
        if not collection:
            raise ValueError("Collection not found")

        if data_source.type == DataSourceType.UPLOAD:
            loader = FileLoader(
                data_source=data_source,
                collection=collection,
            )
            await loader.reload()
        else:
            raise NotImplementedError("Not implemented")


async def create_data_source(
    data_source: KnowledgeDataSourceCreate, db_session: AsyncSession
) -> KnowledgeDataSource:
    data_source_attributes = {
        **data_source.model_dump(exclude_none=True),
        "status": DataSourceStatus.READY,
    }
    data_source_id = await db_session.scalar(
        insert(StorageDataSource)
        .values(data_source_attributes)
        .on_conflict_do_update(
            index_elements=[StorageDataSource.id],
            set_=data_source_attributes,
        )
        .returning(StorageDataSource.id)
    )
    if not data_source_id:
        raise Exception("Failed to create data_source")
    return KnowledgeDataSource(id=str(data_source_id), **data_source_attributes)



from typing import Annotated

from litestar import Controller, Request, get, post
from litestar.di import Provide
from litestar.exceptions import ValidationException
from litestar.plugins.pydantic import PydanticDTO
from litestar.response import ServerSentEvent
from sparkden.models.shared import BaseModel, OffsetPagination
from sparkden.models.task import CreateTaskParams, SendMessageParams, Task, TaskItem
from sparkden.services.task import TaskService
from sparkden.shared.utils import dto_config, snake_case_dict


class GetTaskResponse(BaseModel):
    task: Task | None


CreateTaskParamsDTO = PydanticDTO[Annotated[CreateTaskParams, dto_config()]]
SendMessageParamsDTO = PydanticDTO[Annotated[SendMessageParams, dto_config()]]
TaskDTO = PydanticDTO[Annotated[Task, dto_config()]]
GetTaskResponseDTO = PydanticDTO[Annotated[GetTaskResponse, dto_config()]]
ListTasksResponseDTO = PydanticDTO[Annotated[OffsetPagination[TaskItem], dto_config()]]


def get_task_service(request: Request) -> TaskService:
    return TaskService(user_id=str(request.user.id))


class TaskController(Controller):
    path = "/tasks"
    dependencies = {
        "task_service": Provide(get_task_service, sync_to_thread=True),
    }

    @get("/", return_dto=ListTasksResponseDTO)
    async def list_tasks(
        self, task_service: TaskService, limit: int = 30, offset: int = 0
    ) -> OffsetPagination[TaskItem]:
        return await task_service.list_tasks(limit=limit, offset=offset)

    @post("/", dto=CreateTaskParamsDTO, return_dto=TaskDTO)
    async def create_task(
        self, task_service: TaskService, data: CreateTaskParams
    ) -> Task:
        return await task_service.create_task(
            title=data.title, assistant_id=data.assistant_id, task_id=data.task_id
        )

    @get("/{task_id:str}", return_dto=GetTaskResponseDTO)
    async def get_task(
        self, task_service: TaskService, task_id: str, assistant_id: str
    ) -> GetTaskResponse:
        task = await task_service.get_task(assistant_id=assistant_id, task_id=task_id)
        return GetTaskResponse(task=task)

    @post("/{task_id:str}/run", dto=SendMessageParamsDTO)
    async def run_task(
        self, task_service: TaskService, task_id: str, data: SendMessageParams
    ) -> ServerSentEvent:
        message = data.message or data.message_part
        if message is None:
            raise ValidationException(
                status_code=400, detail="param message or message_part is required"
            )

        return ServerSentEvent(
            task_service.run_task(
                assistant_id=data.assistant_id,
                task_id=task_id,
                message=message,
                state_delta=snake_case_dict(data.state_delta)
                if data.state_delta
                else None,
                edit_message_id=data.edit_message_id,
            )
        )



from enum import StrEnum
from typing import Callable, NotRequired, Sequence, TypedDict

from google.adk.agents import BaseAgent
from google.adk.plugins.base_plugin import BasePlugin
from litestar import Controller
from litestar.handlers import ASGIRouteHandler
from pydantic import ConfigDict

from sparkden.models.knowledge import DataSourceType
from sparkden.models.shared import BaseModel, ExtraInfoMixin, base_model_config
from sparkden.services.knowledge.data_loaders.data_fetchers.base import (
    BaseDataFetcher,
)
from sparkden.services.knowledge.data_loaders.data_splitters.base import (
    BaseDataSplitter,
)


class ToolResponseStatus(StrEnum):
    PENDING = "pending"
    SUCCESS = "success"
    ERROR = "error"


class ToolResponse[ResultT](TypedDict):
    """The base tool response."""

    result: NotRequired[ResultT]
    """The result of the tool."""

    status: NotRequired[ToolResponseStatus]
    """The status of the tool response."""

    error: NotRequired[str]
    """The error message of the tool response."""


class UserApprovalResult[ItemT](BaseModel):
    """The user approval result."""

    approved: bool
    """Whether the user approved the request."""

    feedback: str
    """The feedback for the user approval."""

    modified_item: ItemT | None = None
    """A modified item provided on approval as result, or on rejection as a suggestion."""


class ProgressItemStatus(StrEnum):
    COMPLETED = "completed"
    RUNNING = "running"
    PENDING = "pending"


class KnowledgeDataSourceDefinition(ExtraInfoMixin, BaseModel):
    model_config = ConfigDict(
        **base_model_config,
        arbitrary_types_allowed=True,
    )
    id: str
    type: DataSourceType
    data_fetcher: BaseDataFetcher | None = None
    data_splitter: BaseDataSplitter | None = None


AssistantApiRoute = type[Controller] | ASGIRouteHandler


class Assistant(BaseModel):
    model_config = ConfigDict(
        **base_model_config,
        arbitrary_types_allowed=True,
    )

    id: str
    disabled: bool = False
    root_agent: BaseAgent
    api_routes: Sequence[AssistantApiRoute] | None = None
    agent_plugins: list[BasePlugin] = []
    callbacks: dict[str, Callable] | None = None
    data_sources: list[KnowledgeDataSourceDefinition] | None = None
    sequence: int = 9999



from typing import cast

import pymupdf
from sparkden.models.knowledge import (
    FetchDataResult,
    FetchDataType,
    FileObject,
    KnowledgeDataSource,
)
from sparkden.shared.minio import S3Error, get_minio_client
from sparkden.shared.pymupdf_rag import to_markdown

from .base import BaseDataFetcher


class FileDataFetcher(BaseDataFetcher):
    def fetch(self, data_source: KnowledgeDataSource) -> FetchDataResult:
        file = cast(FileObject, data_source.get_extra_info("file"))
        if not file:
            file = self.fetch_object(data_source)
        if not file:
            raise ValueError("File not found")
        if file.content_type == "text/plain":
            return self._parse_text_file(file)
        elif file.content_type == "application/pdf":
            return self._parse_pdf_file(file, data_source.collection_id)
        else:
            raise ValueError(f"Unsupported file type: {file.content_type}")

    def fetch_object(
        self, data_source: KnowledgeDataSource, object_name: str = "original"
    ) -> FileObject | None:
        file_hash = data_source.get_extra_info("file_hash")
        if not file_hash:
            return None

        object_name = f"{file_hash}/{object_name}"
        response = None
        try:
            minio_client = get_minio_client()
            response = minio_client.get_object(data_source.collection_id, object_name)
            fallback_content_type = response.headers["content-type"]
            content_type = data_source.get_extra_info(
                "file_type", fallback_content_type
            )
            file = FileObject(
                name=object_name,
                content=response.data,
                content_type=content_type,
            )
        except S3Error as e:
            if e.code == "NoSuchKey":
                return None
            raise e
        finally:
            if response:
                response.close()
                response.release_conn()

        return file

    def fetch_parsed_file(self, data_source: KnowledgeDataSource) -> FileObject | None:
        return self.fetch_object(data_source, object_name="parsed")

    @staticmethod
    def _parse_text_file(file: FileObject) -> FetchDataResult:
        return FetchDataResult(
            data=file.content.decode("utf-8"),
            data_type=FetchDataType.TEXT,
            extracted_files=[
                FileObject(
                    name=f"{file.hash}/original",
                    content=file.content,
                    content_type=file.content_type,
                )
            ],
        )

    @staticmethod
    def _parse_pdf_file(file: FileObject, collection_id: str) -> FetchDataResult:
        doc = pymupdf.open(stream=file.content, filetype="pdf")
        extracted_images: list[FileObject] = []

        def extract_image(image_data: bytes, image_name: str) -> str:
            extracted_images.append(
                FileObject(
                    name=image_name,
                    content=image_data,
                    content_type="image/png",
                )
            )
            return f"/images/{collection_id}/{image_name}"

        markdown = to_markdown(
            doc,
            filename=file.hash,
            write_images=extract_image is not None,
            on_write_image=extract_image,
            image_format="png",
        )

        return FetchDataResult(
            data=markdown,
            data_type=FetchDataType.MARKDOWN,
            extracted_files=[
                FileObject(
                    name=f"{file.hash}/original",
                    content=file.content,
                    content_type=file.content_type,
                ),
                FileObject(
                    name=f"{file.hash}/parsed",
                    content=markdown.encode("utf-8"),
                    content_type="text/markdown",
                ),
                *extracted_images,
            ],
        )



from sparkden.db.schema import KnowledgeCollection as StorageCollection
from sparkden.models.knowledge import (
    KnowledgeCollection,
    RetrievalMode,
    ScoredDataChunk,
)
from sparkden.services.base import BaseService
from sparkden.shared.minio import get_minio_client
from sparkden.shared.pg import get_session_maker
from sqlalchemy import select
from sqlalchemy.dialects.postgresql import insert

from .embeddings import Embeddings
from .vector_store import (
    Distance,
    Field,
    Filter,
    VectorParams,
    VectorStore,
)

default_vector_params = VectorParams(size=1024, distance=Distance.COSINE)


class KnowledgeCollectionService(BaseService):
    async def list_collections(self) -> list[KnowledgeCollection]:
        async with self.db_session_maker.begin() as db_session:
            query = select(StorageCollection).order_by(
                StorageCollection.created_at.desc()
            )
            collections = await db_session.scalars(query)
            return [
                KnowledgeCollection.from_storage(collection)
                for collection in collections
            ]

    async def create_collection(
        self,
        collection_id: str,
        collection_name: str | None = None,
        vector_params: VectorParams = default_vector_params,
    ) -> None:
        await create_collection(collection_id, collection_name, vector_params)

    async def get_collection(self, collection_id: str) -> KnowledgeCollection | None:
        return await get_collection(collection_id)

    async def search_collection(
        self,
        collection_id: str,
        query: str,
        filter: Filter | None = None,
        k: int = 5,
        retrieval_mode: RetrievalMode = RetrievalMode.DENSE,
        score_threshold: float | None = None,
    ) -> list[ScoredDataChunk]:
        collection = await get_collection(collection_id)
        if not collection:
            raise ValueError(f"Collection not found: {collection_id}")

        vector_store = VectorStore(
            collection_name=collection_id, retrieval_mode=retrieval_mode
        )

        embeddings = Embeddings(dimensions=collection.vector_dimensions)
        return await vector_store.search(query, embeddings, filter, k, score_threshold)


async def get_collection(collection_id: str) -> KnowledgeCollection | None:
    async with get_session_maker().begin() as db_session:
        collection = await db_session.get(StorageCollection, collection_id)
        return KnowledgeCollection.from_storage(collection) if collection else None


async def create_collection(
    collection_id: str,
    collection_name: str | None = None,
    vector_params: VectorParams = default_vector_params,
) -> None:
    try:
        minio_client = get_minio_client()
        if not minio_client.bucket_exists(collection_id):
            minio_client.make_bucket(collection_id)
            print(f"Created MinIO bucket: {collection_id}")
    except Exception as e:
        print(f"Unexpected error initializing MinIO bucket: {e}")
        raise

    try:
        created = await VectorStore.create_collection(
            collection_id, vector_params=vector_params
        )
        if created:
            print(f"Vector store collection created: {collection_id}")

            await VectorStore.create_index(
                collection_id,
                Field(name="sequence_in_data_source", type="integer"),
            )
    except Exception as e:
        print(f"Unexpected error initializing vector store collection: {e}")
        raise

    async with get_session_maker().begin() as db_session:
        collection = await db_session.scalar(
            insert(StorageCollection)
            .values(
                id=collection_id,
                name=collection_name or collection_id,
                vector_dimensions=vector_params.size,
                vector_distance=vector_params.distance,
            )
            .on_conflict_do_nothing(
                index_elements=[StorageCollection.id],
            )
            .returning(StorageCollection)
        )

        if collection:
            print(f"Database collection created: {collection.id}")



from io import BytesIO

from qdrant_client.http.models import (
    FieldCondition,
    Filter,
    MatchValue,
)
from sparkden.models.knowledge import (
    DataChunk,
    FileObject,
)
from sparkden.shared.minio import get_minio_client

from ..embeddings import Embeddings
from ..vector_store import VectorStore
from .base_loader import BaseLoader


class FileLoader(BaseLoader):
    async def load(self, reload: bool = False) -> None:
        data_fetcher = self.get_data_fetcher(self.data_source.type)
        fetch_result = data_fetcher.fetch(self.data_source)

        data_splitter = self.get_data_splitter(fetch_result.data_type)
        chunks = data_splitter.split(fetch_result.data)

        for index, chunk in enumerate(chunks):
            chunk.metadata.update(
                {
                    "sequence_in_data_source": index + 1,
                    "data_source_id": self.data_source.id,
                }
            )

        if reload:
            await self.unload()
        self._save_objects(fetch_result.extracted_files)
        await self._add_to_vector_store(chunks)

    async def unload(self) -> None:
        file_hash = self.data_source.get_extra_info("file_hash")
        if file_hash:
            minio_client = get_minio_client()
            objects_to_delete = minio_client.list_objects(
                self.data_source.collection_id, prefix=file_hash, recursive=True
            )

            self._delete_objects(
                [
                    object.object_name
                    for object in objects_to_delete
                    if object.object_name
                ]
            )

        await self._delete_from_vector_store()

    async def reload(self) -> None:
        await self.load(reload=True)

    def _save_objects(self, objects: list[FileObject]) -> None:
        minio_client = get_minio_client()
        for object in objects:
            minio_client.put_object(
                bucket_name=self.data_source.collection_id,
                object_name=object.name,
                data=BytesIO(object.content),
                length=len(object.content),
                content_type=object.content_type,
            )

    def _delete_objects(self, object_names: list[str]) -> None:
        minio_client = get_minio_client()
        for object_name in object_names:
            minio_client.remove_object(
                bucket_name=self.data_source.collection_id,
                object_name=object_name,
            )

    async def _add_to_vector_store(self, chunks: list[DataChunk]) -> None:
        if not self.collection:
            raise ValueError("Collection is required to get vector dimensions")

        embeddings = Embeddings(dimensions=self.collection.vector_dimensions)
        vector_store = VectorStore(
            collection_name=self.collection.id,
            retrieval_mode=self.collection.retrieval_mode,
        )

        await vector_store.add_chunks(chunks, embeddings=embeddings)

    async def _delete_from_vector_store(self) -> None:
        vector_store = VectorStore(
            collection_name=self.data_source.collection_id,
        )

        filter = Filter(
            must=[
                FieldCondition(
                    key="data_source_id",
                    match=MatchValue(value=self.data_source.id),
                ),
            ]
        )

        await vector_store.delete_chunks(filter)



import os

from dotenv import load_dotenv
from litestar import Litestar
from litestar.config.compression import CompressionConfig
from litestar.config.cors import CORSConfig
from litestar.config.csrf import CSRFConfig
from litestar.stores.redis import RedisStore

from .assistants import assistants as assistant_registry
from .assistants.models import Assistant
from .db.init import init_db
from .routes.assistants import AssistantController
from .routes.knowledge import KnowledgeController
from .routes.security.auth import session_auth, session_backend_config
from .routes.security.csrf import csrf_token
from .routes.tasks import TaskController
from .shared import getenv
from .shared.pg import dispose_engine

# env variables auto loaded in docker compose
if os.getenv("REDIS_URL") is None:
    load_dotenv()


def create_app(assistants: list[Assistant] = []) -> Litestar:
    for assistant in assistants:
        assistant_registry.register(assistant)

    redis_store = RedisStore.with_client(url=getenv("REDIS_URL"))

    return Litestar(
        route_handlers=[
            csrf_token,
            AssistantController,
            TaskController,
            KnowledgeController,
            *assistant_registry.get_api_routes(),
        ],
        stores={
            session_backend_config.store: redis_store.with_namespace("sessions"),
        },
        lifespan=[*assistant_registry.get_callbacks("lifespan")],
        on_app_init=[
            session_auth.on_app_init,
            *assistant_registry.get_callbacks("on_app_init"),
        ],
        on_startup=[init_db, *assistant_registry.get_callbacks("on_startup")],
        on_shutdown=[dispose_engine, *assistant_registry.get_callbacks("on_shutdown")],
        csrf_config=CSRFConfig(
            secret=getenv("CSRF_SECRET"),
        ),
        cors_config=CORSConfig(
            allow_origins=[url.strip() for url in getenv("CLIENT_URL", "*").split(",")],
            allow_credentials=True,
        ),
        compression_config=CompressionConfig(backend="gzip"),
        openapi_config=None,
        debug=getenv("DEBUG", "false") == "true",
    )



from abc import ABC, abstractmethod
from typing import TYPE_CHECKING, Any

if TYPE_CHECKING:
    from sparkden.models.knowledge import DataChunk


class BaseDataSplitter(ABC):
    @abstractmethod
    def split(self, data: Any) -> list["DataChunk"]:
        pass



from google.adk.agents.base_agent import BaseAgent
from google.adk.agents.callback_context import CallbackContext
from google.adk.models.llm_response import LlmResponse
from google.adk.plugins.base_plugin import BasePlugin
from sparkden.assistants.models import ProgressItemStatus


class ProgressInfoPlugin(BasePlugin):
    def __init__(self):
        super().__init__(name="progress_info")

    async def before_agent_callback(
        self, *, agent: BaseAgent, callback_context: CallbackContext
    ) -> None:
        progress_info = callback_context.state.get("progress_info") or {}
        progress_info[callback_context.agent_name] = ProgressItemStatus.RUNNING
        callback_context.state["progress_info"] = progress_info

    async def after_model_callback(
        self, *, callback_context: CallbackContext, llm_response: LlmResponse
    ) -> LlmResponse | None:
        progress_info = callback_context.state.get("progress_info")
        if progress_info:
            progress_info.pop(callback_context.agent_name, None)
            callback_context.state["progress_info"] = progress_info
        else:
            callback_context.state["progress_info"] = None



from uuid import uuid4

from langchain_text_splitters import RecursiveCharacterTextSplitter
from sparkden.models.knowledge import DataChunk

from .base import BaseDataSplitter


class TextSplitter(BaseDataSplitter):
    def split(self, data: str) -> list[DataChunk]:
        recursive_splitter = RecursiveCharacterTextSplitter(
            chunk_size=1000, chunk_overlap=200
        )

        chunks = recursive_splitter.create_documents([data])

        return [
            DataChunk(
                id=str(uuid4().hex),
                content=chunk.page_content,
                metadata=chunk.metadata,
            )
            for chunk in chunks
        ]
